package com.hzya.frame.sysnew.comparison.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.sysnew.comparison.dao.IComparisonDao;
import com.hzya.frame.sysnew.comparison.dao.IComparisonDetailsDao;
import com.hzya.frame.sysnew.comparison.entity.ComparisonDetailsEntity;
import com.hzya.frame.sysnew.comparison.entity.ComparisonEntity;
import com.hzya.frame.sysnew.comparison.service.IComparisonService;
import com.hzya.frame.sysnew.generalData.dao.IGeneralDataDao;
import com.hzya.frame.uuid.UUIDUtils;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import sun.java2d.pipe.AAShapePipe;

import java.util.*;

@Service(value = "comparisonServiceImpl")
public class ComparisonServiceImpl extends BaseService<ComparisonEntity, String> implements IComparisonService {

    private IComparisonDao comparisonDao;

    @Autowired
    private IComparisonDetailsDao comparisonDetailsDao;

    @Autowired
    public void setGeneralDataDao(IComparisonDao dao) {
        this.comparisonDao = dao;
        this.dao = dao;
    }

    //分页查询档案对照数据
    @Override
    public JsonResultEntity queryEntityPage(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (jsonObject == null) {
            return BaseResult.getFailureMessageEntity("数据为空，请先传递数据");
        }
        if (!checkStr(jsonObject.getString("mdmCode"))) {
            return BaseResult.getFailureMessageEntity("主数据编码为空");
        }
        ComparisonEntity comparisonEntity = jsonObject.toJavaObject(ComparisonEntity.class);
        List<ComparisonEntity> comparisonEntities=new ArrayList<>();
        try {
            //根据主数据编码查询表名称，字段名称等信息 10001
            comparisonEntities = comparisonDao.queryComparison(comparisonEntity);
        }catch (Exception e){
            logger.info("根据主数据编码查询信息错误:{}",e.getMessage());
            return BaseResult.getFailureMessageEntity("查询失败",e.getMessage());
        }
        Map<String, Object> map = new HashMap<>();//将主表数据存到map中
        if (comparisonEntities != null && comparisonEntities.size() > 0) {
            for (ComparisonEntity entity : comparisonEntities) {
                //遍历主表下面的所有key,字段名
                for (String key : jsonObject.keySet()) {
                    if (entity.getDbType().equals("1")) {
                        comparisonEntity.setDbName(entity.getDbName());
                        if(!"mdmCode".equals(key)){
                            map.put(key, jsonObject.getString(key));
                        }
                    }
                }
            }
            comparisonEntity.setMap(map);
            try {
                List<ComparisonEntity> comparisonEntitiePages = comparisonDao.queryComparisonPage(comparisonEntity);
                PageInfo pageInfo = new PageInfo(comparisonEntitiePages);
                return BaseResult.getSuccessMessageEntity("查询成功", pageInfo);
            } catch (Exception e) {
                logger.info("查询档案对照错误:{}", e.getMessage());
                return BaseResult.getFailureMessageEntity("查询失败", e.getMessage());
            }
        }
        return BaseResult.getFailureMessageEntity("查询失败","请检查传递参数");
    }

    //新增档案对照数据
    @Override
    public JsonResultEntity saveEntity(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (jsonObject == null) {
            return BaseResult.getFailureMessageEntity("数据为空，请先传递数据");
        }
        if (!checkStr(jsonObject.getString("mdmCode"))) {
            return BaseResult.getFailureMessageEntity("主数据编码为空");
        }
        if (!checkStr(jsonObject.getString("rulePrefix"))) {
            return BaseResult.getFailureMessageEntity("规则为空");
        }
        ComparisonEntity comparisonEntity = jsonObject.toJavaObject(ComparisonEntity.class);
        ComparisonDetailsEntity comparisonDetailsEntity=new ComparisonDetailsEntity();
            List<ComparisonEntity> comparisonEntities=new ArrayList<>();
            try {
                //根据主数据编码查询表名称，字段名称等信息 10001
                comparisonEntities = comparisonDao.queryComparison(comparisonEntity);
            }catch (Exception e){
                logger.info("根据主数据编码查询信息错误:{}",e.getMessage());
                return BaseResult.getFailureMessageEntity("保存失败",e.getMessage());
            }
            String main = jsonObject.getString("main");//获取主表数据
            JSONArray jsonArray = jsonObject.getJSONArray("details");//获取子表数据
            JSONObject jsonObjectMain = JSONObject.parseObject(main);
            if(jsonObjectMain != null ) {
                Map<String, Object> map = new HashMap<>();//将主表数据存到map中
                Map<String, Object> mapDetails = new HashMap<>();//将子表数据存到map中
                if (comparisonEntities != null && comparisonEntities.size() > 0) {
                    for (ComparisonEntity entity : comparisonEntities) {
                        //遍历主表下面的所有key,字段名
                        for (String key : jsonObjectMain.keySet()) {
                            if (entity.getDbType().equals("1")) {
                                comparisonEntity.setDbName(entity.getDbName());
                                map.put(key, jsonObjectMain.getString(key));
                            }
                        }
                    }
                    if (jsonArray != null && jsonArray.size() > 0) {
                        for (Object object : jsonArray) {
                            JSONObject jsonObjectDetails = (JSONObject) JSON.toJSON(object);
                            for (ComparisonEntity entity : comparisonEntities) {
                                //遍历主表下面的所有key,字段名
                                for (String key : jsonObjectDetails.keySet()) {
                                    if (entity.getDbType().equals("2")) {
                                        comparisonDetailsEntity.setDbName(entity.getDbName());
                                        mapDetails.put(key, jsonObjectDetails.getString(key));
                                    }
                                }
                            }
                        }
                    }
                    String uuid = UUIDUtils.getUUID();
                    map.put("id", uuid);
                    map.put("create_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                    map.put("modify_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                    map.put("org_id", "0");
                    map.put("create_time", new Date());
                    map.put("modify_time", new Date());
                    map.put("sts", "Y");
                    comparisonEntity.setMap(map);
                    //生成单据规则
                    comparisonEntity.setRulePrefix(jsonObject.getString("rulePrefix"));
                    try {
                        ComparisonEntity comparisonEntityRule = comparisonDao.queryComparisonRule(comparisonEntity);
                        map.put("document_rule", comparisonEntityRule.getDocumentRule());//单据规则
                        map.put("document_rule_num", comparisonEntityRule.getDocumentRuleNum());//单据规则流水号
                    } catch (Exception e) {
                        logger.info("保存档案对照时候生成单据规则错误:{}", e.getMessage());
                        return BaseResult.getFailureMessageEntity("保存失败", e.getMessage());
                    }
                    try {
                        //保存主表数据
                        ComparisonEntity comparisonEntityOne = comparisonDao.saveComparison(comparisonEntity);
                        //保存子表数据
                        if (mapDetails != null && mapDetails.size() > 0) {
                            mapDetails.put("id", UUIDUtils.getUUID());
                            mapDetails.put("formmain_id", uuid);
                            mapDetails.put("create_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                            mapDetails.put("modify_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                            mapDetails.put("org_id", "0");
                            mapDetails.put("create_time", new Date());
                            mapDetails.put("modify_time", new Date());
                            mapDetails.put("sts", "Y");
                            comparisonDetailsEntity.setMapDetails(mapDetails);
                            ComparisonDetailsEntity comparisonDetailsEntityOne = comparisonDetailsDao.saveComparisonDetails(comparisonDetailsEntity);
                            comparisonEntityOne.setComparisonDetailsEntity(comparisonDetailsEntityOne);
                        }
                        return BaseResult.getSuccessMessageEntity("保存成功", comparisonEntityOne);
                    } catch (Exception e) {
                        logger.info("保存档案对照时候错误:{}", e.getMessage());
                        return BaseResult.getFailureMessageEntity("保存失败", e.getMessage());
                    }
                }
            }
        return BaseResult.getFailureMessageEntity("查询失败","请检查传递参数");
    }

    //更新档案对照数据
    @Override
    public JsonResultEntity updateEntity(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (jsonObject == null) {
            return BaseResult.getFailureMessageEntity("数据为空，请先传递数据");
        }
        if (!checkStr(jsonObject.getString("mdmCode"))) {
            return BaseResult.getFailureMessageEntity("主数据编码为空");
        }
        ComparisonEntity comparisonEntity = jsonObject.toJavaObject(ComparisonEntity.class);
        ComparisonDetailsEntity comparisonDetailsEntity=new ComparisonDetailsEntity();
        JSONObject jsonObjectMain=jsonObject.getJSONObject("main");//获取主表数据
        JSONArray jsonArray = jsonObject.getJSONArray("details");//获取子表数据
        if(!checkStr(jsonObjectMain.getString("id"))){
            return BaseResult.getFailureMessageEntity("更新时候主表id不能为空");
        }
        List<ComparisonEntity> comparisonEntities=new ArrayList<>();
        //根据主数据编码查询表名称，字段名称等信息 10001
        try {
           comparisonEntities = comparisonDao.queryComparison(comparisonEntity);
        }catch (Exception e){
            logger.info("根据主数据编码查询信息错误:{}",e.getMessage());
            return BaseResult.getFailureMessageEntity("更新失败",e.getMessage());
        }
        Map<String,Object> map=new HashMap<>();//将主表数据存到map中
        Map<String,Object> mapDetails=new HashMap<>();//将子表数据存到map中
        if(comparisonEntities !=null && comparisonEntities.size()>0) {
            for (ComparisonEntity entity : comparisonEntities) {
                //遍历主表下面的所有key,字段名
                for (String key : jsonObjectMain.keySet()) {
                    if (entity.getDbType().equals("1")) {
                        comparisonEntity.setDbName(entity.getDbName());
                        map.put(key, jsonObjectMain.getString(key));
                    }
                }
            }
            if (jsonArray != null && jsonArray.size() > 0) {
                for (Object object : jsonArray) {
                    JSONObject jsonObjectDetails = (JSONObject) JSON.toJSON(object);
                    for (ComparisonEntity entity : comparisonEntities) {
                        //遍历主表下面的所有key,字段名
                        for (String key : jsonObjectDetails.keySet()) {
                            if (entity.getDbType().equals("2")) {
                                comparisonDetailsEntity.setDbName(entity.getDbName());
                                mapDetails.put(key, jsonObjectDetails.getString(key));
                            }
                        }
                    }
                }
            }
            map.put("create_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
            map.put("modify_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
            map.put("org_id", "0");
            map.put("modify_time", new Date());
            map.put("sts", "Y");
            comparisonEntity.setMap(map);
            comparisonEntity.setId(jsonObjectMain.getString("id"));
        /*//生成单据规则
        comparisonEntity.setRulePrefix(jsonObject.getString("rulePrefix"));
        try {
            ComparisonEntity comparisonEntityRule = comparisonDao.queryComparisonRule(comparisonEntity);
            map.put("document_rule",comparisonEntityRule.getDocumentRule());//单据规则
            map.put("document_rule_num",comparisonEntityRule.getDocumentRuleNum());//单据规则流水号
        }catch (Exception e){
            logger.info("更新档案对照时候生成单据规则错误:{}",e.getMessage());
            return BaseResult.getFailureMessageEntity("更新失败",e.getMessage());
        }*/
            try {
                //保存主表数据
                Integer integer = comparisonDao.updateComparisonByType(comparisonEntity);
                if (integer > 0) {
                    //保存子表数据
                    if (mapDetails != null && mapDetails.size() > 0) {
                        mapDetails.put("create_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                        mapDetails.put("modify_user_id", "c796fd9ba4c9f5ff3cc2fa41a040e443");
                        mapDetails.put("org_id", "0");
                        mapDetails.put("modify_time", new Date());
                        mapDetails.put("sts", "Y");
                        comparisonDetailsEntity.setFormmainId(jsonObjectMain.getString("id"));
                        comparisonDetailsEntity.setMapDetails(mapDetails);
                        comparisonDetailsDao.updateComparisonDetailsByType(comparisonDetailsEntity);
                        return BaseResult.getSuccessMessageEntity("更新成功", integer);
                    }
                }else{
                    return BaseResult.getFailureMessageEntity("更新失败", "请查看id是否在表中存在");
                }
            } catch (Exception e) {
                logger.info("更新档案对照时候错误:{}", e.getMessage());
                return BaseResult.getFailureMessageEntity("更新失败", e.getMessage());
            }
        }
        return BaseResult.getFailureMessageEntity("查询失败","请检查传递参数");
    }

    //删除当初对照数据
    @Override
    public JsonResultEntity deleteEntity(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (jsonObject == null) {
            return BaseResult.getFailureMessageEntity("数据为空，请先传递数据");
        }
        if (!checkStr(jsonObject.getString("mdmCode"))) {
            return BaseResult.getFailureMessageEntity("主数据编码为空");
        }
        ComparisonEntity comparisonEntity = jsonObject.toJavaObject(ComparisonEntity.class);
        ComparisonDetailsEntity comparisonDetailsEntity=new ComparisonDetailsEntity();
        JSONObject jsonObjectMain=jsonObject.getJSONObject("main");//获取主表数据
        if(!checkStr(jsonObjectMain.getString("id"))){
            return BaseResult.getFailureMessageEntity("删除时候主表id不能为空");
        }
        List<ComparisonEntity> comparisonEntities=new ArrayList<>();
        //根据主数据编码查询表名称，字段名称等信息 10001
        try {
            comparisonEntities = comparisonDao.queryComparison(comparisonEntity);
        }catch (Exception e){
            logger.info("根据主数据编码查询信息错误:{}",e.getMessage());
            return BaseResult.getFailureMessageEntity("删除失败",e.getMessage());
        }
        Map<String,Object> map=new HashMap<>();
        if(comparisonEntities !=null && comparisonEntities.size()>0) {
            for (ComparisonEntity entity : comparisonEntities) {
                //遍历主表下面的所有key,字段名
                for (String key : jsonObjectMain.keySet()) {
                    if (entity.getDbType().equals("1")) {
                        comparisonEntity.setDbName(entity.getDbName());
                        map.put(key, jsonObjectMain.getString(key));
                    }
                    if(entity.getDbType().equals("2")){
                        comparisonDetailsEntity.setDbName(entity.getDbName());
                    }
                }
            }
            comparisonEntity.setId(jsonObjectMain.getString("id"));
            comparisonEntity.setModify_user_id("c796fd9ba4c9f5ff3cc2fa41a040e443");
            comparisonEntity.setMap(map);
            try {
                Integer integer = comparisonDao.deleteComparison(comparisonEntity);
                if(integer>0){
                    comparisonDetailsEntity.setFormmainId(jsonObjectMain.getString("id"));
                    comparisonDetailsEntity.setModify_user_id("c796fd9ba4c9f5ff3cc2fa41a040e443");
                     comparisonDetailsDao.deleteComparisonDetails(comparisonDetailsEntity);
                     return BaseResult.getSuccessMessageEntity("删除成功",integer);
                }else{
                    return BaseResult.getFailureMessageEntity("删除失败","请查看id是否在表中存在");
                }
            }catch (Exception e){
                logger.info("删除档案对照数据错误:{}",e.getMessage());
                return BaseResult.getFailureMessageEntity("删除失败",e.getMessage());
            }
        }
        return BaseResult.getFailureMessageEntity("查询失败","请检查传递参数");
    }
}
